#! /bin/sh

###########################################################
#
# @file: config/saga-run.sh(.in)
#
# @description:
#   can be used to run programs linked against SAGA from within 
#   the SAGA source tree, if everything built ok, but is not yet 
#   installed.  If install was done already (i.e. if saga-run is 
#   available in $SAGA_LOCATION/bin), then the program is run against
#   SAGA_LOCATION.
#
# Copyright (c) 2005-2007 Andre Merzky (andre@merzky.net)
# This file is part of the SAGA Engine.
#
# Contributed by Andre Merzky   <andre@merzky.net>
#
# FIXME: need to add adaptor dirs, and need to fix adaptor inis
#
###########################################################

### LICENSE ###

saga_root="@SAGA_ROOT@"
saga_location="/usr"
saga_ostype="SAGA_LINUX"
boost_location="/usr"
sqlite3_location=""
postgresql_location=""

if ! test "x$SAGA_LOCATION" = "x"; then
  saga_location=$SAGA_LOCATION
fi

if ! test "x$BOOST_LOCATION" = "x"; then
  boost_location=$BOOST_LOCATION
fi

if ! test "x$SQLITE3_LOCATION" = "x"; then
  sqlite3_location=$SQLITE3_LOCATION
fi

if ! test "x$POSTGRESQL_LOCATION" = "x"; then
  postgresql_location=$POSTGRESQL_LOCATION
fi


if test -x $saga_location/bin/saga-run.sh; then
  # after install, we use saga from install tree in SAGA_LOCATION
  type=install

  PATH=$saga_location/bin:$PATH
  LD_LIBRARY_PATH=$boost_location/lib:$sqlite3_location/lib:$postgresql_location/lib:$saga_location/lib:$LD_LIBRARY_PATH

else
  # by default, however, use saga from source tree in SAGA_ROOT
  type=source

  if ! test -s $SAGA_ROOT; then
    echo "cannot find SAGA sources ($saga_root)"
    exit -1
  fi

  LD_LIBRARY_PATH=$boost_location/lib:$sqlite3_location/lib:$postgresql_location/lib:$saga_root/saga/impl/engine:$LD_LIBRARY_PATH
  
  for package in `ls -d $saga_root/saga/impl/packages/*/`; do
    LD_LIBRARY_PATH=$package:$LD_LIBRARY_PATH
  done
  
  for adaptor in `ls -d $saga_root/adaptors/*/*/`; do
    LD_LIBRARY_PATH=$adaptor:$LD_LIBRARY_PATH
  done

fi

export LD_LIBRARY_PATH

if test "x$saga_ostype" = "xSAGA_APPLE"; then
  DYLD_LIBRARY_PATH=$LD_LIBRARY_PATH
  export DYLD_LIBRARY_PATH
fi


usage()
{
  cat <<EOF

  Usage: saga-run.sh <program> [arguments ...]

EOF
  exit $1
}

if test $# -eq 0;
then
  usage 1 1>&2
fi

LOG=$HOME/.saga/log
if test -e $LOG; then
  echo                         >>       $LOG
  echo "---------------------" >>       $LOG
  date                         >>       $LOG
  echo "saga-run.sh: start $@" >>       $LOG
  echo                         >>       $LOG
  $@                           | tee -a $LOG
  echo                         >>       $LOG
  date                         >>       $LOG
  echo "---------------------" >>       $LOG
  echo                         >>       $LOG
else
  $@
fi

